@@ -10,6 +10,7 @@ import os
 import datetime
 from stream import serializer
 
+
 def connect_debug():
     return stream.connect(
         u'ahj2ndz7gsan',
@@ -34,7 +35,7 @@ class ClientTest(TestCase):
         self.aggregated3 = aggregated3
         self.topic1 = topic1
         self.flat3 = flat3
-        
+
     def test_heroku(self):
         url = 'https://thierry:pass@getstream.io/?site=1'
         os.environ['STREAM_URL'] = url
@@ -42,7 +43,7 @@ class ClientTest(TestCase):
         self.assertEqual(client.api_key, 'thierry')
         self.assertEqual(client.api_secret, 'pass')
         self.assertEqual(client.site_id, '1')
-        
+
     def test_heroku_overwrite(self):
         url = 'https://thierry:pass@getstream.io/?site=1'
         os.environ['STREAM_URL'] = url
@@ -61,20 +62,34 @@ class ClientTest(TestCase):
         activities = self.user1.get(limit=1)['results']
         self.assertEqual(activities[0]['id'], activity_id)
 
+    def test_add_activity_to(self):
+        activity_data = {
+            'actor': 1, 'verb': 'tweet', 'object': 1,
+            'to': ['user:pyto1']
+        }
+        response = self.user1.add_activity(activity_data)
+        feed = client.feed('user:pyto1')
+        activity_id = response['id']
+        activities = feed.get(limit=1)['results']
+        self.assertEqual(activities[0]['id'], activity_id)
+
     def test_remove_activity(self):
         activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
         activity_id = self.user1.add_activity(activity_data)['id']
         self.user1.remove_activity(activity_id)
         activities = self.user1.get(limit=1)['results']
         self.assertNotEqual(activities[0]['id'], activity_id)
-        
+
     def test_remove_activity_by_foreign_id(self):
-        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1, 'foreign_id': 'tweet:10'}
+        activity_data = {
+            'actor': 1, 'verb': 'tweet', 'object': 1, 'foreign_id': 'tweet:10'}
         activity_id = self.user1.add_activity(activity_data)['id']
         self.user1.remove_activity(foreign_id='tweet:10')
         activities = self.user1.get(limit=1)['results']
         self.assertNotEqual(activities[0]['id'], activity_id)
-        
+        # verify this doesnt raise an error, but fails silently
+        self.user1.remove_activity(foreign_id='tweet:unknowandmissing')
+
     def test_add_activities(self):
         activity_data = [
             {'actor': 1, 'verb': 'tweet', 'object': 1},
@@ -86,6 +101,28 @@ class ClientTest(TestCase):
         get_activity_ids = [a['id'] for a in activities]
         self.assertEqual(get_activity_ids, activity_ids[::-1])
 
+    def test_add_activities_to(self):
+        to = ['user:pyto2', 'user:pyto3']
+        activity_data = [
+            {'actor': 1, 'verb': 'tweet', 'object': 1, 'to': to},
+            {'actor': 2, 'verb': 'watch', 'object': 2, 'to': to},
+        ]
+        response = self.user1.add_activities(activity_data)
+        activity_ids = [a['id'] for a in response['activities']]
+        activities = self.user1.get(limit=2)['results']
+        get_activity_ids = [a['id'] for a in activities]
+        self.assertEqual(get_activity_ids, activity_ids[::-1])
+        # test first target
+        feed = client.feed('user:pyto2')
+        activities = feed.get(limit=2)['results']
+        get_activity_ids = [a['id'] for a in activities]
+        self.assertEqual(get_activity_ids, activity_ids[::-1])
+        # test second target
+        feed = client.feed('user:pyto3')
+        activities = feed.get(limit=2)['results']
+        get_activity_ids = [a['id'] for a in activities]
+        self.assertEqual(get_activity_ids, activity_ids[::-1])
+
     def test_follow(self):
         actor_id = random.randint(10, 100000)
         activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
@@ -97,6 +134,18 @@ class ClientTest(TestCase):
         activity_id_found = activity['id'] if activity is not None else None
         self.assertEqual(activity_id_found, activity_id)
 
+    def test_follow_private(self):
+        feed = client.feed('secret:py1')
+        actor_id = random.randint(10, 100000)
+        activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
+        activity_id = feed.add_activity(activity_data)['id']
+        self.aggregated2.follow('secret:py1')
+        time.sleep(10)
+        activities = self.aggregated2.get(limit=3)['results']
+        activity = self._get_first_aggregated_activity(activities)
+        activity_id_found = activity['id'] if activity is not None else None
+        self.assertEqual(activity_id_found, activity_id)
+
     def test_flat_follow(self):
         activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
         activity_id = self.user1.add_activity(activity_data)['id']
@@ -130,6 +179,62 @@ class ClientTest(TestCase):
         activity_id_found = activity['id'] if activity is not None else None
         self.assertNotEqual(activity_id_found, activity_id)
 
+    def test_empty_followings(self):
+        asocial = client.feed('user:asocialpython')
+        followings = asocial.following()
+        self.assertEqual(followings['results'], [])
+
+    def test_get_followings(self):
+        social = client.feed('user:psocial')
+        social.follow('user:apy')
+        social.follow('user:bpy')
+        social.follow('user:cpy')
+        followings = social.following(offset=0, limit=2)
+        self.assertEqual(len(followings['results']), 2)
+        self.assertEqual(followings['results'][0]['feed_id'], 'user:psocial')
+        self.assertEqual(followings['results'][0]['target_id'], 'user:cpy')
+        followings = social.following(offset=1, limit=2)
+        self.assertEqual(len(followings['results']), 2)
+        self.assertEqual(followings['results'][0]['feed_id'], 'user:psocial')
+        self.assertEqual(followings['results'][0]['target_id'], 'user:bpy')
+
+    def test_empty_followers(self):
+        asocial = client.feed('user:asocialpython')
+        followers = asocial.following()
+        self.assertEqual(len(followers['results']), 0)
+        self.assertEqual(followers['results'], [])
+
+    def test_get_followers(self):
+        social = client.feed('user:psocial')
+        client.feed('user:spammy1').follow('user:psocial')
+        client.feed('user:spammy2').follow('user:psocial')
+        client.feed('user:spammy3').follow('user:psocial')
+        followers = social.followers(offset=0, limit=2)
+        self.assertEqual(len(followers['results']), 2)
+        self.assertEqual(followers['results'][0]['feed_id'], 'user:spammy3')
+        self.assertEqual(followers['results'][0]['target_id'], 'user:psocial')
+        followers = social.followers(offset=1, limit=2)
+        self.assertEqual(len(followers['results']), 2)
+        self.assertEqual(followers['results'][0]['feed_id'], 'user:spammy2')
+        self.assertEqual(followers['results'][0]['target_id'], 'user:psocial')
+
+    def test_empty_do_i_follow(self):
+        social = client.feed('user:psocial')
+        social.follow('user:apy')
+        social.follow('user:bpy')
+        followings = social.following(feeds=['user:missingpy'])
+        self.assertEqual(len(followings['results']), 0)
+        self.assertEqual(followings['results'], [])
+
+    def test_do_i_follow(self):
+        social = client.feed('user:psocial')
+        social.follow('user:apy')
+        social.follow('user:bpy')
+        followings = social.following(feeds=['user:apy'])
+        self.assertEqual(len(followings['results']), 1)
+        self.assertEqual(followings['results'][0]['feed_id'], 'user:psocial')
+        self.assertEqual(followings['results'][0]['target_id'], 'user:apy')
+
     def test_get(self):
         activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
         activity_id = self.user1.add_activity(activity_data)['id']
@@ -161,38 +266,39 @@ class ClientTest(TestCase):
                           self.user1.add_activity(activity_data))
 
     def test_complex_field(self):
-        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1, 'participants': ['Tommaso', 'Thierry']}
+        activity_data = {'actor': 1, 'verb': 'tweet',
+                         'object': 1, 'participants': ['Tommaso', 'Thierry']}
         response = self.user1.add_activity(activity_data)
         activity_id = response['id']
         activities = self.user1.get(limit=1)['results']
         self.assertEqual(activities[0]['id'], activity_id)
         self.assertEqual(activities[0]['participants'], ['Tommaso', 'Thierry'])
-        
+
     def assertDatetimeAlmostEqual(self, a, b):
-        difference = abs(a-b)
+        difference = abs(a - b)
         if difference > datetime.timedelta(milliseconds=1):
             self.assertEqual(a, b)
 
     def assertClearlyNotEqual(self, a, b):
-        difference = abs(a-b)
+        difference = abs(a - b)
         if difference < datetime.timedelta(milliseconds=1):
             raise ValueError('the dates are too close')
-        
+
     def test_uniqueness(self):
         '''
         In order for things to be considere unique they need:
         a.) The same time and activity data
         b.) The same time and foreign id
         '''
-        now = datetime.datetime.now(tzlocal())
         utcnow = datetime.datetime.utcnow()
-        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1, 'time': utcnow}
+        activity_data = {
+            'actor': 1, 'verb': 'tweet', 'object': 1, 'time': utcnow}
         response = self.user1.add_activity(activity_data)
         response = self.user1.add_activity(activity_data)
         activities = self.user1.get(limit=2)['results']
         self.assertDatetimeAlmostEqual(activities[0]['time'], utcnow)
         self.assertClearlyNotEqual(activities[1]['time'], utcnow)
-        
+
     def test_uniqueness_topic(self):
         '''
         In order for things to be considere unique they need:
@@ -205,28 +311,32 @@ class ClientTest(TestCase):
         # add the same activity twice
         now = datetime.datetime.now(tzlocal())
         tweet = 'My Way %s' % random.randint(10, 100000)
-        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1, 'time': now, 'tweet': tweet}
+        activity_data = {
+            'actor': 1, 'verb': 'tweet', 'object': 1, 'time': now, 'tweet': tweet}
         self.topic1.add_activity(activity_data)
         self.user1.add_activity(activity_data)
         # verify that flat3 contains the activity exactly once
         response = self.flat3.get(limit=3)
         activity_tweets = [a.get('tweet') for a in response['results']]
         self.assertEqual(activity_tweets.count(tweet), 1)
-        
+
     def test_uniqueness_foreign_id(self):
         now = datetime.datetime.now(tzlocal())
-        utcnow = datetime.datetime.utcnow()
-        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1, 'foreign_id': 'tweet:11', 'time': now}
+        utcnow = (now - now.utcoffset()).replace(tzinfo=None)
+        activity_data = {'actor': 1, 'verb': 'tweet',
+                         'object': 1, 'foreign_id': 'tweet:11', 'time': now}
         response = self.user1.add_activity(activity_data)
-        activity_data = {'actor': 2, 'verb': 'tweet', 'object': 3, 'foreign_id': 'tweet:11', 'time': now}
+        activity_data = {'actor': 2, 'verb': 'tweet',
+                         'object': 3, 'foreign_id': 'tweet:11', 'time': now}
         response = self.user1.add_activity(activity_data)
         activities = self.user1.get(limit=2)['results']
-        # the second post should have overwritten the first one (because they had same id)
+        # the second post should have overwritten the first one (because they
+        # had same id)
         self.assertEqual(activities[0]['object'], '3')
         self.assertEqual(activities[0]['foreign_id'], 'tweet:11')
         self.assertDatetimeAlmostEqual(activities[0]['time'], utcnow)
         self.assertNotEqual(activities[1]['foreign_id'], 'tweet:11')
-        
+
     def test_missing_actor(self):
         activity_data = {'verb': 'tweet', 'object':
                          1, 'debug_example_undefined': 'test'}
@@ -243,14 +353,12 @@ class ClientTest(TestCase):
             'tfq2sdqpj9g446sbv653x3aqmgn33hsn8uzdc9jpskaw8mj6vsnhzswuwptuj9su'
         )
         self.assertRaises(ValueError, lambda: self.c.feed('user1'))
-        
+
     def test_serialization(self):
         today = datetime.date.today()
         now = datetime.datetime.now()
-        data = dict(string='string', float=0.1, int=1, date=today, datetime=now)
+        data = dict(
+            string='string', float=0.1, int=1, date=today, datetime=now)
         serialized = serializer.dumps(data)
         loaded = serializer.loads(serialized)
         self.assertEqual(data, loaded)
-        
-        
-        
